# SRM Two Underlying Instruments Test.R
# Illustrate risk statistics for two instruments
# rmarkdown::render("SRM Two Underlying Instruments Test.R", "word_document")
rm(list = ls())  # Take out the Environment "trash"
cat("\014") # Clear Console, making error checking easier.
while (!is.null(dev.list()))  dev.off() # Clear old plots
par(family = 'Times New Roman') # Globally set fonts for graphs
# Libraries
#  date - functions for handling dates
#  openxlsx - manipulate spreadsheet files
Packages <- c("date", "zoo", "openxlsx", "stats", "moments", "quantmod", "date") 
if(length(setdiff(Packages, rownames(installed.packages()))) > 0) {
  install.packages(setdiff(Packages, rownames(installed.packages())))
} # Make sure libraries are installed on this computer
lapply(Packages, library, character.only=TRUE) # Load and attach libraries
rm(Packages)
#
# Import large data files so we trim with start and end date
#
StartDate <- as.Date('2010-12-31') # '2020-03-18'
EndDate <- as.Date('2023-06-07')
RollingWindow = 130 # 1/2 year (260 trading days per year)
COLOR <- TRUE
if(COLOR){
  Colr <- c("Blue", "Red", "Green", "Purple")
} else {
  Colr <- c("Black", "Black", "Black", "Black")
}
# Instrument #1
TCKR <- 'SPY'
mTitle1t <- paste0("SP 500 Index ETF")
# TCKR <- 'WY'
# mTitle1t <- paste0("Weyerhaeuser")
# TCKR <- 'CUT'
# mTitle1t <- paste0("MSCI Global Timber ETF")
# TCKR <- 'WEN'
# mTitle1t <- paste0("Wendy's")
mTitleX <- paste0(" (Daily Data, 130 Day Rolling Window)")
mTitle1 <- paste0(mTitle1t, mTitleX)
source('Instrument 1.R')
dfResults1 = dfStockRisk1
# Instrument #2 -- can reference different files, but same format
TCKR <- '^VIX'
mTitle2t <- paste0("Volatility Index (VIX)")
# TCKR <- 'WOOD'
# mTitle2t <- paste0("Global Timber and Forestry ETF")
# TCKR <- 'SPY'
# mTitle2t <- paste0("SP 500 Index ETF")
# TCKR <- 'GPK'
# mTitle2t <- paste0("Graphic Packaging")
# TCKR <- 'WRK'
# mTitle2t <- paste0("Westrock")
# TCKR <- 'WFG'
# mTitle2t <- paste0("West Fraser")
# TCKR <- 'IP'
# mTitle2t <- paste0("International Paper")
# TCKR <- 'MCD'
# mTitle2t <- paste0("McDonalds")
mTitle2 <- paste0(mTitle2t, mTitleX)
source('Instrument 2.R') # Use with same File, but different contracts
dfResults2 = dfStockRisk2
mTitle = paste0(mTitle1t, " and ", mTitle2t, mTitleX)
#
# Price
#
yTitle = "Adjusted Price (Normalized to $1 Initially)"
X = as.date(dfResults1$Date)
P1 = as.double(dfResults1$Price[1])
P2 = as.double(dfResults2$Price[1])
Y1 = as.data.frame(dfResults1$Price/P1)
Y2 = as.data.frame(dfResults2$Price/P2)

Date1 <- as.data.frame(dfResults1$Date)
Date2 <- as.data.frame(dfResults2$Date)
df1 <- cbind(Date1,Y1)
names(df1) <- c("Date", "Price1")
df2 <- cbind(Date2,Y2)
names(df2) <- c("Date", "Price2")
dfPrice <- merge(df1, df2, by="Date", all=FALSE)

MaxValue = max(X); MinValue = min(X)
xlim1 = c(1:2); xlim1[1] = MinValue; xlim1[2] = MaxValue
MaxValue = max(Y1, Y2); MinValue = min(Y1, Y2)
ylim1 = c(1:2); ylim1[1] = MinValue; ylim1[2] = MaxValue
xTitle = "Date"
legtxt = c(mTitle1t, mTitle2t)
lTitle = "Parameter"
plot(X, Y1, type = "b", main = paste0(mTitle1t, " and ", mTitle2t),
  xlab = xTitle, ylab = yTitle, col = Colr[1], xlim = xlim1, 
  ylim = ylim1, pch = 1, cex = 0.5)
lines(X, Y2, type = "b", col =Colr[2], xlim = xlim1, ylim = ylim1,
  pch = 2, cex = 0.5)
legend("topleft", legtxt, cex = 0.75, lwd = c(1, 1), lty = c(1, 1), 
  col = c(Colr[1],Colr[2]), pch = c(1,2), bty = "n", title = lTitle)




#
# Mean (CC)
#
yTitle = "Mean (CC)"
X = as.date(dfResults1$Date)
XY = as.data.frame(X)
XY$Y1 = dfResults1$CCMean
XY$Y2 = dfResults2$CCMean
XY = na.omit(XY)
X <- XY$X
Y1 <- XY$Y1
Y2 <- XY$Y2
Y3 = Y1 - Y2
MaxValue = max(X); MinValue = min(X)
xlim1 = c(1:2); xlim1[1] = MinValue; xlim1[2] = MaxValue
MaxValue = max(Y1, Y2, na.rm = TRUE); MinValue = min(Y1, Y2, na.rm = TRUE)
ylim1 = c(1:2); ylim1[1] = MinValue; ylim1[2] = MaxValue
xTitle = "Date"
legtxt = c(mTitle1t, mTitle2t)
lTitle = "Parameter"
plot(X, Y1, type = "b", main = mTitle,
  xlab = xTitle, ylab = yTitle, col = Colr[1], xlim = xlim1, 
  ylim = ylim1, pch = 1, cex = 0.5)
lines(X, Y2, type = "b", col =Colr[2], xlim = xlim1, ylim = ylim1,
  pch = 2, cex = 0.5)
legend("bottomleft", legtxt, cex = 0.75, lwd = c(1, 1), lty = c(1, 1), 
  col = c(Colr[1],Colr[2]), pch = c(1,2), bty = "n", title = lTitle)
yTitle = 'Difference in Mean'
MaxValue = max(Y3, na.rm = TRUE); MinValue = min(Y3, na.rm = TRUE)
ylim1 = c(1:2); ylim1[1] = MinValue; ylim1[2] = MaxValue
plot(X, Y3, type = "b", main = mTitle,
  xlab = xTitle, ylab = yTitle, col = Colr[1], xlim = xlim1, 
  ylim = ylim1, pch = 1, cex = 0.5)
#
# Standard Deviation (CC)
#
yTitle = "Standard Deviation (CC)"
X = as.date(dfResults1$Date)
XY = as.data.frame(X)
XY$Y1 = dfResults1$CCStDev
XY$Y2 = dfResults2$CCStDev
XY = na.omit(XY)
X <- XY$X
Y1 <- XY$Y1
Y2 <- XY$Y2
Y3 = Y1 - Y2
MaxValue = max(X); MinValue = min(X)
xlim1 = c(1:2); xlim1[1] = MinValue; xlim1[2] = MaxValue
MaxValue = max(Y1, Y2, na.rm = TRUE); MinValue = min(Y1, Y2, na.rm = TRUE)
ylim1 = c(1:2); ylim1[1] = MinValue; ylim1[2] = MaxValue
xTitle = "Date"
legtxt = c(mTitle1t, mTitle2t)
lTitle = "Parameter"
plot(X, Y1, type = "b", main = mTitle,
  xlab = xTitle, ylab = yTitle, col = Colr[1], xlim = xlim1, 
  ylim = ylim1, pch = 1, cex = 0.5)
lines(X, Y2, type = "b", col =Colr[2], xlim = xlim1, ylim = ylim1,
  pch = 2, cex = 0.5)
legend("topleft", legtxt, cex = 0.75, lwd = c(1, 1), lty = c(1, 1), 
  col = c(Colr[1],Colr[2]), pch = c(1,2), bty = "n", title = lTitle)
yTitle = 'Difference in Standard Deviation'
MaxValue = max(Y3, na.rm = TRUE); MinValue = min(Y3, na.rm = TRUE)
ylim1 = c(1:2); ylim1[1] = MinValue; ylim1[2] = MaxValue
plot(X, Y3, type = "b", main = mTitle,
  xlab = xTitle, ylab = yTitle, col = Colr[1], xlim = xlim1, 
  ylim = ylim1, pch = 1, cex = 0.5)
#
# Skewness (CC)
#
yTitle = "Skewness (CC)"
X = as.date(dfResults1$Date)
XY = as.data.frame(X)
XY$Y1 = dfResults1$CCSkewness
XY$Y2 = dfResults2$CCSkewness
XY = na.omit(XY)
X <- XY$X
Y1 <- XY$Y1
Y2 <- XY$Y2
Y3 = Y1 - Y2
MaxValue = max(X); MinValue = min(X)
xlim1 = c(1:2); xlim1[1] = MinValue; xlim1[2] = MaxValue
MaxValue = max(Y1, Y2, na.rm = TRUE); MinValue = min(Y1, Y2, na.rm = TRUE)
ylim1 = c(1:2); ylim1[1] = MinValue; ylim1[2] = MaxValue
xTitle = "Date"
legtxt = c(mTitle1t, mTitle2t)
lTitle = "Parameter"
plot(X, Y1, type = "b", main = mTitle,
  xlab = xTitle, ylab = yTitle, col = Colr[1], xlim = xlim1, 
  ylim = ylim1, pch = 1, cex = 0.5)
lines(X, Y2, type = "b", col =Colr[2], xlim = xlim1, ylim = ylim1,
  pch = 2, cex = 0.5)
legend("topleft", legtxt, cex = 0.75, lwd = c(1, 1), lty = c(1, 1), 
  col = c(Colr[1],Colr[2]), pch = c(1,2), bty = "n", title = lTitle)
yTitle = 'Difference in Skewness'
MaxValue = max(Y3, na.rm = TRUE); MinValue = min(Y3, na.rm = TRUE)
ylim1 = c(1:2); ylim1[1] = MinValue; ylim1[2] = MaxValue
plot(X, Y3, type = "b", main = mTitle,
  xlab = xTitle, ylab = yTitle, col = Colr[1], xlim = xlim1, 
  ylim = ylim1, pch = 1, cex = 0.5)
#
# Kurtosis (CC)
#
yTitle = "Kurtosis (CC)"
X = as.date(dfResults1$Date)
XY = as.data.frame(X)
XY$Y1 = dfResults1$CCKurtosis
XY$Y2 = dfResults2$CCKurtosis
XY = na.omit(XY)
X <- XY$X
Y1 <- XY$Y1
Y2 <- XY$Y2
Y3 = Y1 - Y2
MaxValue = max(X); MinValue = min(X)
xlim1 = c(1:2); xlim1[1] = MinValue; xlim1[2] = MaxValue
MaxValue = max(Y1, Y2, na.rm = TRUE); MinValue = min(Y1, Y2, na.rm = TRUE)
ylim1 = c(1:2); ylim1[1] = MinValue; ylim1[2] = MaxValue
xTitle = "Date"
legtxt = c(mTitle1t, mTitle2t)
lTitle = "Parameter"
plot(X, Y1, type = "b", main = mTitle,
  xlab = xTitle, ylab = yTitle, col = Colr[1], xlim = xlim1, 
  ylim = ylim1, pch = 1, cex = 0.5)
lines(X, Y2, type = "b", col =Colr[2], xlim = xlim1, ylim = ylim1,
  pch = 2, cex = 0.5)
legend("topleft", legtxt, cex = 0.75, lwd = c(1, 1), lty = c(1, 1), 
  col = c(Colr[1],Colr[2]), pch = c(1,2), bty = "n", title = lTitle)
yTitle = 'Difference in Kurtosis'
MaxValue = max(Y3, na.rm = TRUE); MinValue = min(Y3, na.rm = TRUE)
ylim1 = c(1:2); ylim1[1] = MinValue; ylim1[2] = MaxValue
plot(X, Y3, type = "b", main = mTitle,
  xlab = xTitle, ylab = yTitle, col = Colr[1], xlim = xlim1, 
  ylim = ylim1, pch = 1, cex = 0.5)

